<?php
// Software released under the General Public License (version 2 or later), available at
// http://www.gnu.org/copyleft/gpl.html

require_once 'approach/astronomy/solarsystem/SolarSystemConstants.php';
require_once 'approach/astronomy/solarsystem/SolarFramesConstants.php';
require_once 'spacetime/SpaceConstants.php';
/******************************************************************************
Facade for ephemeris computation ; code external to this package should use this class.

The role of this class is :
- recieve a client request
- choose the engine(s) to perform the computations
- order the computations
- handle the problems and throw appropriate exceptions
- finalize results (coord transformation, convert to the right units)
- return results

@designPattern Facade
@license  GPL
@author   <a href="http://www.tig12.net">Thierry Graff</a>
@history  2007.12.12, 20h50 : Creation
@history  2008.05.07 : Transformed to a facade
********************************************************************************/

class Ephemeris{
  
  //********************* compute ******************************
  /**
    Main entry point of this package, to order computation
    @param $jd The julian day of the computation
    @param $what Constant or array of constants designating solar system bodies that need to be computed
    @param $options Associative array of key / value pairs
                  Possible keys are :
                  - 'precision' : Precision of the computations for positions, in arc seconds
                  - 'frame' : Frame in which the coordinates should be expressed, using constants of {@link SolarFrameConstants} 
                  - 'sphereCart' : Expression mode of the coordinates (spherical, cartesian), using constants of {@link SpaceConstants} 
                  - 'coordUnits' : units to express the coordinates. --- not implemented yet ---
                  - 'velocities' : Boolean indicating if the velocities should be computed too
                  - 'velocitiesPrecision' : precision of the computations for velocities --- suppress ?????
  */
  public static function compute($jd, $what, $options = array()){
    //
    // parameter defaults
    if(!isset($options['precision'])) $options['precision'] = 1; // arc seconds
    if(!isset($options['frame'])) $options['frame'] = SolarFramesConstants::EC;
    if(!isset($options['sphereCart'])) $options['sphereCart'] = SpaceConstants::SPHERICAL;
    if(!isset($options['coordUnits'])) $options['coordUnits'] = ''; // not implemented
    if(!isset($options['velocities'])) $options['velocities'] = false;
    if(!isset($options['velocitiesPrecision'])) $options['velocitiesPrecision'] = 1; // arc second per time second
    //
    // choose the engine performing the computation
    $engineClass = (isset($options['engine']) ? $options['engine'] : self::getEngine($jd, $options['precision']));
    //
    // order the computation
    $eval = "\$res = $engineClass::compute(\$jd, \$what, \$options);";
    eval($eval);
    return $res;
  }// end compute
  
  
  // *************************** getEngine ***************************
  /**
    Computes the class name of the ephemeris engine to use for a given precision and julian day.
    Requires the file containing the engine, and returns the class name
    @param $jd
    @param $precision
  */
  public static function getEngine($jd, $precision){
    if($precision > 1){
      return 'Meeus1';
    }
    //Moshier
if(false){
    require_once dirname(__FILE__) . '/moshier/aa56.php';
    return 'aa56';
}
    //Swiss ephemeris
    require_once dirname(__FILE__) . '/sweph/Sweph.php';
    return 'Sweph';
  }// end getEngine
  
  
}//end class

?>
