<?php
// Software released under the General Public License (version 2 or later), available at
// http://www.gnu.org/copyleft/gpl.html

require_once("approach/astronomy/solarsystem/SolarSystemConstants.php");
require_once("approach/astronomy/solarsystem/ephemeris/EphemerisEngine.php");
require_once("approach/astrology/western/DomificationConstants.php");

/******************************************************************************
Utilisation of SwissEphemeris coputations

@license  GPL
@author   <a href="http://www.tig12.net">Thierry Graff</a>
@history  2008.06.01, 20h19 : Creation
********************************************************************************/

class Sweph implements EphemerisEngine{
  
  /**
    Correspondance between JeTheme domification constants
    and sweph constants used in arguments
  */
  private static $match_arg_domification = array(
    DomificationConstants::AEQUELIS => 'A',
    DomificationConstants::CAMPANUS => 'C',
    DomificationConstants::HORIZON => 'H',
    DomificationConstants::KOCH => 'K',
    DomificationConstants::PORPHYRY => 'O',
    DomificationConstants::PLACIDUS => 'P',
    DomificationConstants::REGIOMONTANUS => 'R',
    DomificationConstants::VEHLOW => 'V',
    DomificationConstants::AXIAL => 'X',
  );
  
  /**
    Correspondance between JeTheme planet constants
    and sweph constants used in arguments 
  */
  private static $match_arg_planets = array(
// p main factors as above, plus main asteroids DEFGHI
// h ficticious factors J..X
// z hypothetical body, with number given in -xz
// s minor planet, with MPC number given in -xs
// a all factors
    SolarSystemConstants::SUN => '0',
    SolarSystemConstants::MERCURY => '2',
    SolarSystemConstants::VENUS => '3',
    SolarSystemConstants::EARTH => 'C',
    SolarSystemConstants::MOON => '1',
    SolarSystemConstants::MARS => '4',
    SolarSystemConstants::JUPITER => '5',
    SolarSystemConstants::SATURN => '6',
    SolarSystemConstants::URANUS => '7',
    SolarSystemConstants::NEPTUNE => '8',
    SolarSystemConstants::PLUTO => '9',
    //
    SolarSystemConstants::MEAN_LUNAR_NODE => 'm',
    SolarSystemConstants::TRUE_LUNAR_NODE => 't',
    SolarSystemConstants::OBLIQUITY => 'o',
    SolarSystemConstants::NUTATION => 'n',
    SolarSystemConstants::MEAN_LUNAR_APOGEE => 'A', // black moon
    SolarSystemConstants::OSCULTATING_LUNAR_APOGEE => 'B', // true black moon ?
    //
    SolarSystemConstants::CHIRON => 'D',
    SolarSystemConstants::PHOLUS => 'E',
    SolarSystemConstants::CERES => 'F',
    SolarSystemConstants::PALLAS => 'G',
    SolarSystemConstants::JUNO => 'H',
    SolarSystemConstants::VESTA => 'I',
    SolarSystemConstants::CUPIDO => 'J',
    SolarSystemConstants::HADES => 'K',
    SolarSystemConstants::ZEUS => 'L',
    SolarSystemConstants::KRONOS => 'M',
// N Apollon 
// O Admetos 
// P Vulkanus 
// Q Poseidon 
// R Isis (Sevin) 
// S Nibiru (Sitchin) 
// T Harrington 
// U Leverrier's Neptune
// V Adams' Neptune
// W Lowell's Pluto
// X Pickering's Pluton
// Y Vulcann
// Z White Moon
  );
  
  /** Correspondance between JeTheme constants and sweph output */
  private static $match_output = array(
    'Sun' => SolarSystemConstants::SUN,
    'Mercury' => SolarSystemConstants::MERCURY,
    'Venus' => SolarSystemConstants::VENUS,
    'Moon' => SolarSystemConstants::MOON,
    'Mars' => SolarSystemConstants::MARS,
    'Jupiter' => SolarSystemConstants::JUPITER,
    'Saturn' => SolarSystemConstants::SATURN,
    'Uranus' => SolarSystemConstants::URANUS,
    'Neptune' => SolarSystemConstants::NEPTUNE,
    'Pluto' => SolarSystemConstants::PLUTO,
    //
    'mean Node' => SolarSystemConstants::MEAN_LUNAR_NODE,
    'true Node' => SolarSystemConstants::TRUE_LUNAR_NODE,
//    '' => SolarSystemConstants::OBLIQUITY,
//    '' => SolarSystemConstants::NUTATION,
    'mean Apogee' => SolarSystemConstants::MEAN_LUNAR_APOGEE, // black moon
    'osc. Apogee' => SolarSystemConstants::OSCULTATING_LUNAR_APOGEE, // true black moon ?
    //
    'Ceres' => SolarSystemConstants::CERES,
    'Pallas' => SolarSystemConstants::PALLAS,
    'Juno' => SolarSystemConstants::JUNO,
    'Vesta' => SolarSystemConstants::VESTA,
    //
    'Chiron' => SolarSystemConstants::CHIRON,
    'Pholus' => SolarSystemConstants::PHOLUS,
//    '' => SolarSystemConstants::CUPIDO,
//    '' => SolarSystemConstants::HADES,
//    '' => SolarSystemConstants::ZEUS,
//    '' => SolarSystemConstants::KRONOS,
    //
    'house  1' => DomificationConstants::H1,
    'house  2' => DomificationConstants::H2,
    'house  3' => DomificationConstants::H3,
    'house  4' => DomificationConstants::H4,
    'house  5' => DomificationConstants::H5,
    'house  6' => DomificationConstants::H6,
    'house  7' => DomificationConstants::H7,
    'house  8' => DomificationConstants::H8,
    'house  9' => DomificationConstants::H9,
    'house 10' => DomificationConstants::H10,
    'house 11' => DomificationConstants::H11,
    'house 12' => DomificationConstants::H12,
  );

  
  //********************* compute ******************************
  /**
    Computations using Swiss Ephemeris
    @param $options Associative array of key / value pairs
                  keys can be :
                    - 'longitude' : longitude, in decimal degrees
                    - 'latitude' : latitude, in decimal degrees
                    - 'domification_system' : constant of {@link DomificationConstants}.
  */
  public static function compute($jd, $what, $options=array()){
    //
    // prepare the command to call binary executable
    //
    $os = 'linux'; // todo : compute
    $swe_path = G::get('dir_data') . '/approach/astronomy/ephemeris/sweph';
    $command = $swe_path . '/swetest_' . $os;
    //
    // build argument list, depending on this function's arguments
    //
    $args = '';
    // planets to compute
//    $args .= ' -pp';
    $args .= ' -pd';
    // julian day
    $args .= ' -j' . $jd;
    $args .= ' -ut'; // necassary to transmit julian day
    // houses
    $args .= ' -house3.5,44.5,P';
    // ephemeris dir
    $args .= ' -edir' . $swe_path;
    // no header
    $args .= ' -head';
    // return format;
    $args .= ' -fPlbr';
//echo "<br/>" . $command . $args;
    //
    // execute swiss ephem
    //
    exec($command . $args, $output);
//echo "<pre>"; print_r($output); echo "</pre>";
    //
    // Parse the output and fill returned array
    //
    // pattern for a line containing one coordinate
    $p1 = '/^(.*?)(-?\d+\.\d+)\s*$/';
    // pattern for a line containing three coordinates
    $p3 = '/^(.*?)\s+(-?\d+\.\d+)\s+(-?\d+\.\d+)\s+(-?\d+\.\d+)/';
    foreach($output as $line){
//echo "<br/>line = $line";
      // try line with 3 coordinates
      preg_match($p3, $line, $matches);
      if(count($matches) == 5){
        if(isset(self::$match_output[$matches[1]])){
          $object = self::$match_output[$matches[1]];
          $res[$object] = array(
            'x0' => $matches[4],
            'x1' => $matches[2],
            'x2' => $matches[3],
          );
//echo "<pre>res[$object] : "; print_r($res[$object]); echo "</pre>";
        }
      }
      else{// try line with 1 coordinate
        preg_match($p1, $line, $matches);
//echo "<pre>matches : "; print_r($matches); echo "</pre>";
        if(count($matches) == 3 && isset(self::$match_output[trim($matches[1])])){
          $object = self::$match_output[trim($matches[1])];
          $res[$object] = $matches[2];
//echo "<pre>res[$object] : "; print_r($res[$object]); echo "</pre>";
        }
      }// end for
    }
    return $res;
  }// end compute
  
  
  // *************************** parseCoordinates ***************************
  /**
    Parses a piece of line returned by sweph, and returns an array in jetheme format
    ex : <code>280.1533511   0.0000569    0.983266267</code>
    returns <code></code>array('x0'=>0.983266267, 'x1'=>280.1533511, 'x2'=>0.0000569);
    @param $str Piece of line to parse
  */
  private static function parseCoordinates($str){
    $tmp = explode(' ', $str);
echo "<pre>tmp = "; print_r($tmp); echo "</pre>";
    return array(
      'x0' => trim($tmp[2]),
      'x1' => trim($tmp[0]),
      'x2' => trim($tmp[1])
    );
  }// end 
  
}//end class



?>
