<?php
// Software released under the General Public License (version 3 or later), available at
// http://www.gnu.org/copyleft/gpl.html

require_once 'approach/astrology/western/AstrologyUtils.php';

/******************************************************************************
Conversion of longitudes between formats used by JeTheme.
See <a href='todo'>Longitude and latitude formats</a>.
Limitation : arc seconds are limited to integer.

@license  GPL
@author   <a href="http://www.tig12.net">Thierry Graff</a>
@history  2008.05.20, 12h42 : Creation
********************************************************************************/
class Longitudes{
  
  //********************* convert ******************************
  /**
    Converts longitudes between the formats used in JeTheme
    @param $long The longitude, expressed in <code>$inputFormat</code>
    @param $inputFormat Format in which $long is expressed
                  Can be 'decimal', 'db', 'user'
    @param $outputFormat Format of $long
                  Can be 'decimal', 'db', 'user'
    @return $long converted in <code>$outputFormat</code>
    @throws Exception if $inputFormat or $outputFormat are incorrect
                   Exception if longitude is not valid
    @todo Finish implementation
  */
  public static function convert($long, $inputFormat, $outputFormat){
    if(!in_array($inputFormat, array('decimal', 'db', 'user')))
      throw new InvalidArgumentException("Invalid parameter \$inputFormat : <b>$inputFormat</b>"); 
    if(!in_array($outputFormat, array('decimal', 'db', 'user')))
      throw new InvalidArgumentException("Invalid parameter \$outputFormat : <b>$outputFormat</b>");
    $long = trim($long);
    switch($inputFormat){
      case 'user' :
        switch($outputFormat){
          case 'db' : return self::user2db($long); break;
          case 'decimal' : return self::user2decimal($long); break;
        }
      break;
      case 'db' :
        // not implemented
      break;
      case 'decimal' :
        // not implemented
      break;
    }
  }// end convert
  
  
  // *************************** toString ***************************
  /**
    Returns a formatted string containing the longitude expressed with sign and degrees, as used in astrology.
    @param $options Associative array of key /value pairs
                  Note : several options are common with {@link uFormatStrings::toDMS()} 
                  Keys can be :
                  - 'sign_display' : 'full', 'abbr' or 'none' ; to indicate how the sign is displayed
                  Default : 'none'
                  - 'format' : can be 'D', 'DM', or 'DMS', to have a string with degrees only, or degrees minutes, or degrees minutes seconds
                  Default : 'DMS'
                  - 'echo_last_symbol' : Boolean ; If true the last symbol (° for degrees ; ' for minutes ; " for seconds) is appended to the string
                  ex with format = 'DM', and $x = 12.5 : returns <code>12°30'</code> if true ; returns <code>12°30</code> if false
                  Default : true
                  - 'degree_symbol' : The symbol used for degrees (in general '°' or '&amp;deg;')
                  Default : '°'
  */
  public static function toString($long, $options=array()){
    // parameter stuff
    if(!isset($options['sign_display'])) $options['sign_display'] = 'abbr';
    if(!isset($options['lang'])) $options['lang'] = G::get('user_language');
    //
    $long = uMaths::mod360($long);
    $decPart = $long - floor($long);
    $signIdx = AstrologyUtils::getSign($long);
    $deg = $long%30 + $decPart;
    if($options['sign_display'] == 'abbr')
      $strSign = ' ' . AstrologyUtils::getSignAbbreviation($signIdx, $options['lang']);
    else if($options['sign_display'] == 'full')
      $strSign = ' ' . AstrologyUtils::getSignName($signIdx, $options['lang']);
    else
      $strSign = '';
    return uFormatStrings::toDMS($deg, $options) . $strSign;
  }// end toString
  
  
  // *************************** user2db ***************************
  /**
    @param $long Longitude in user format
    @return $long in db format
    @todo not implemented
  */
  private static function user2db($long){
  }// end user2db
  
  
  // *************************** user2decimal ***************************
  /**
    @param $long String Longitude in user format
    @return $long in decimal format
  */
  private static function user2decimal($long){
    $array = self::parseUserLong($long);
    $res = $array[1] + $array[2] / 60 + $array[3] / 3600;
    return $res * ($array[0] == '-' ? -1 : 1);
  }// end user2decimal
  
  
  // =======================================================================
  //                                                                        Auxiliary methods
  // =======================================================================
  
  // *************************** parseUserLong ***************************
  /**
    Parses a longitude expressed in user format.
    @param $long String Longitude in user format
    @return Array containing the parsed longitude
                  Contains 4 elements : sign (+ or -); degrees ; minutes ; seconds
                  If minutes or seconds are not specified, the entries contain empty strings.
    @throws Exception if the longitude is not correctly formatted
  */
  private static function parseUserLong($long){
    $res = array('', '', '', ''); // if minutes or seconds not specified, empty string in the res
    //
    $sign = '';
    $test = substr($long, 0, 1);
    $long2 = $long; // keep $long for the error msg
    if($test == '-' || $test == '+'){
      $sign = $test;
      $long2 = substr($long2, 1);
    }
    //
    $EW = '';
    $test = substr($long2, -1, 1);
    if($test == 'e' || $test == 'E' || $test == 'w' || $test == 'W'){
      $EW = strToUpper($test);
      $long2 = substr($long2, 0, -1);
    }
    //
    if($sign != '' && $EW != '')
      throw new Exception("Wrong format for longitude <b>$long</b> : impossible to specify both <b>$sign</b> and <b>$EW</b>");
    if($sign != ''){
      $res[0] = $sign;
    }
    else{
      if($EW == 'E') $res[0] = '+';
      else if($EW == 'W') $res[0] = '-';
      else  $res[0] = '+'; // $EW = ''
    }
    //
    $pieces = preg_split('/\D/', $long2, -1, PREG_SPLIT_NO_EMPTY);
    if(count($pieces) > 3)
      throw new Exception("Wrong format for longitude <b>$long</b>");
    // degrees
    if(isset($pieces[0])){
      $res[1] = $pieces[0];
    }
    // minutes
    if(isset($pieces[1])){
      if($pieces[1] > 59) throw new Exception("Wrong format for longitude <b>$long</b> : minutes must be inferior to 60");
      $res[2] = $pieces[1];
    }
    // seconds
    if(isset($pieces[2])){
      if($pieces[2] > 59) throw new Exception("Wrong format for longitude <b>$long</b> : seconds must be inferior to 60");
      $res[3] = $pieces[2];
    }
    return $res;
  }// end parseUserLong
  
}//end class

?>
